#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <fcntl.h> //newlib

/* from kos/fs.h */
#define KOS_O_RDONLY	1		/* Read only */
#define KOS_O_RDWR		2		/* Read-write */
#define KOS_O_APPEND	3		/* Append to an existing file */
#define KOS_O_WRONLY	4		/* Write-only */
#define KOS_O_MODE_MASK	7		/* Mask for mode numbers */
#define KOS_O_TRUNC		0x0100		/* Truncate */
#define KOS_O_ASYNC		0x0200		/* Open for asynchronous I/O */
#define KOS_O_DIR		0x1000		/* Open as directory */
#define KOS_O_META		0x2000		/* Open as metadata */

typedef int file_t;

#define	KOS2FD(hnd)	~(hnd)
#define	FD2KOS(fd)	~(fd)

/* some prgram thik fd>=0
but kos fd is allways <0

~hnd means 0->-1  (0x8xxxxxxx)
*/

int open(const char *name,int mode,...)
{
	file_t hnd;
	int kosmode;
	switch (mode&O_ACCMODE) {
	case O_RDONLY: kosmode = KOS_O_RDONLY; break;
	case O_WRONLY: kosmode = KOS_O_WRONLY; break;
	case O_RDWR  : kosmode = KOS_O_RDWR; break;
	default: return -1;
	}
	if (mode&O_APPEND) kosmode = KOS_O_APPEND;
	if (mode&O_TRUNC) kosmode|=KOS_O_TRUNC;

	hnd = fs_open(name,kosmode);
	printf("open(%s,%x)->%x=%x\n",name,mode,kosmode,hnd);
	if (hnd) return KOS2FD(hnd);
	return -1;
}

int read(int fd,void *ptr,size_t len)
{
	return fs_read(FD2KOS(fd),ptr,len);
}

int write(int fd,const void *ptr,size_t len)
{
	return fs_write(FD2KOS(fd),ptr,len);
}

int close(int fd)
{
	fs_close(FD2KOS(fd));
	return 0;
}

off_t lseek(int fd, off_t offset, int whence)
{
	return fs_seek(FD2KOS(fd),offset,whence);
}

int unlink(const char *name)
{
	return fs_unlink(name);
}

DIR *opendir (const char *name)
{
	DIR *ret;
	char fullpath[256];
	if (strcmp(name,".")==0 || strcmp(name,"./")==0) {
		strcpy(fullpath,fs_getwd());
		name = fullpath;
	}
	ret = (DIR*)fs_open(name,KOS_O_DIR|KOS_O_RDONLY);
	printf("opendir:%s = %p\n",name,ret);
	return ret;
}

struct dirent *readdir (DIR *dir)
{
	return (struct dirent*)fs_readdir((file_t)dir);
}

int closedir (DIR *dir)
{
	fs_close((file_t)dir);
	return 0;
}

int mkdir(const char *path,unsigned int mode)
{
	return fs_mkdir(path);
}

int chdir(const char *path)
{
	return fs_chdir(path);
}

char *getcwd(char *buf, size_t size)
{
	char *wd = fs_getwd();
	if (buf==NULL) {
		return strdup(wd); //linux compat
	}
	strncpy(buf,wd,size);
	return buf;
}

/* only support F_OK */
int access(const char *name,int mode)
{
	file_t hnd = fs_open(name,KOS_O_RDONLY);
	if (hnd==0) return -1;
	fs_close(hnd);
	return 0;
}

long filelength(int fd)
{
	return fs_total(FD2KOS(fd));
}

void _exit (int n)
{
  arch_exit();
}

int getpid(void)
{
	return -1;
}

typedef void (*sighandler_t)(int); 
sighandler_t signal(int signum, sighandler_t sighandler)
{
	return (sighandler_t)-1;
}

/* misc */

char *strerror(int errno)
{
	return "strerror";
}

#include <stdarg.h>
/* minimal sscanf only support "%x" */
#if 0
int sscanf(const char *str, const char *fmt,...)
{
	va_list ap;
	if (strcmp(fmt,"%x")) {
		exit(-1);
	}
	va_start(ap,fmt);
	*(va_arg(ap,int*)) = strtol(str,NULL,16);
	va_end(ap);
	return 1;
}
#endif

int snprintf(char *out, size_t n,const char *fmt, ...) {
	va_list args;
	int i;

	va_start(args, fmt);
	i = vsprintf(out, fmt, args);
	va_end(args);

	return i;
}

int vfprintf(FILE *fp, const char *fmt, va_list ap)
{
	char buf[1024];
	int ret = vsprintf(buf, fmt, ap);
	dbgio_printf(buf);
	return ret;
}

void setbuf(FILE *fp,char *buf)
{
	/* do noting */
	
}

#include <time.h>

struct tm* localtime(const time_t *tim_p)
{
	static struct tm tm;
	return localtime_r(tim_p,&tm);
}

#undef getchar
int getchar(void)
{
	return 0;
}
